/*
 * Copyright 2004-2009 Analog Devices Inc.
 *
 * Licensed under the GPL-2 or later.
 *
 */

#ifndef __BFIN_POLL_H
#define __BFIN_POLL_H

#define POLLWRNORM	4 /* POLLOUT */
#define POLLWRBAND	256

#include <asm-generic/poll.h>

#endif
