/* linux/arch/arm/mach-exynos4/include/mach/vmalloc.h
 *
 * Copyright (c) 2010-2011 Samsung Electronics Co., Ltd.
 *		http://www.samsung.com
 *
 * Copyright 2010 Ben Dooks <ben-linux@fluff.org>
 *
 * Based on arch/arm/mach-s5p6440/include/mach/vmalloc.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * EXYNOS4 vmalloc definition
*/

#ifndef __ASM_ARCH_VMALLOC_H
#define __ASM_ARCH_VMALLOC_H __FILE__

#define VMALLOC_END	0xF6000000UL

#endif /* __ASM_ARCH_VMALLOC_H */
